/**********************************************************************+
*
* SuperFP: A program for the reconstruction of the driving potential 
*          of stochastic trajectories, by solving an inverse problem
*          governed by the corresponding Fokker-Planck equation
*
**********************************************************************+
*
* THIS PACKAGE OF PROGRAM FILES IS A SUPPLEMENTAL MATERIAL FOR THE MANUSCRIPT 
* "Fokker-Planck analysis of superresolution microscopy images"
* BY MARIO ANNUNZIATO AND ALFIO BORZI' (NOV. 2023).
*
* THIS SOFTWARE IS EXPERIMENTAL AND IS RELEASED WITH ABSOLUTELY NO WARRANTY.
* IT PROVIDES THE NEED FOR CREATING THE EXECUTABLE FILES THAT HAVE BEEN USED
* BY THE AUTHORS FOR THE NUMERICAL EXPERIMENTS RECONSTRUCTIONS OF SECT. 4 
* AND 5 OF THE PAPER. 
*
* IT INHERITS THE SAME RIGHTS OF THE PUBLISHED PAPER. 
* IF YOU USE THIS CODE OF ANY PART OF IT, YOU ARE PLEASED TO QUOTE THE ARTICLE
* AND THE CODE NAME SuperFP (NOV. 2023)
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/


#include "Chang_Cooper.hpp"

Chang_Cooper::Chang_Cooper(Model* model, Grid* grid_domain)
{
 md = model;





 domain = grid_domain;






 CC_matrix.zeros(domain->NN,domain->NN);
};



void Chang_Cooper::build_Chang_Cooper_matrix()
{


 sp_mat& H = CC_matrix;

 H.zeros();





 mat uux;



 interp2(domain->y,domain->x,*(md->u),domain->y,domain->xx,uux);


 mat uuy;

 interp2(domain->y,domain->x,*(md->u),domain->yy,domain->x,uuy);

 mat Bx = md->val_Bx(domain,&uux);
 mat Cx = md->val_Cx(domain)/domain->dx;



 mat By = md->val_By(domain,&uuy);
 mat Cy = md->val_Cy(domain)/domain->dy;


 mat deltax = delta_CC(Bx,Cx);
 mat deltay = delta_CC(By,Cy);

mat rvalx = domain->alfax * (Cx - deltax % Bx);
mat lvalx = domain->alfax * ((1-deltax) % Bx + Cx);

rvalx.insert_rows(rvalx.n_rows,1);

lvalx.insert_rows(0,1);






mat rvaly = domain->alfay * (Cy - deltay % By);
mat lvaly = domain->alfay * ((1-deltay) % By + Cy);







rvaly.insert_cols(rvaly.n_cols,1);







lvaly.insert_cols(0,1);

H.diag() = vectorise(rvalx);

H.diag() += vectorise(lvalx);

H.diag() += vectorise(rvaly);
H.diag() += vectorise(lvaly);

lvalx = shift(lvalx,-1,0);

H.diag(1) -= resize(vectorise(lvalx),lvalx.n_elem-1,1);







lvaly.shed_col(0);

H.diag(domain->Nx) -= vectorise(lvaly);

H.diag(-1) -= resize(vectorise(rvalx),rvalx.n_elem-1,1);

rvaly.shed_col(rvaly.n_cols-1);



H.diag(-domain->Nx) -= vectorise(rvaly);



};


mat Chang_Cooper::delta_CC(mat B, mat C)
{

mat w = B/C;
mat delta(size(w));


uvec iw = vectorise(abs(w) > 1e-4);

uvec iw1 = find(iw==1);
uvec iw0 = find(iw==0);


delta(iw1) = 1/w(iw1) - 1/(exp(w(iw1))-1);


delta(iw0)=0.5*(1-w(iw0)/2);

return delta;

};
