/**********************************************************************+
*
* SuperFP: A program for the reconstruction of the driving potential 
*          of stochastic trajectories, by solving an inverse problem
*          governed by the corresponding Fokker-Planck equation
*
**********************************************************************+
*
* THIS PACKAGE OF PROGRAM FILES IS A SUPPLEMENTAL MATERIAL FOR THE MANUSCRIPT 
* "Fokker-Planck analysis of superresolution microscopy images"
* BY MARIO ANNUNZIATO AND ALFIO BORZI' (NOV. 2023).
*
* THIS SOFTWARE IS EXPERIMENTAL AND IS RELEASED WITH ABSOLUTELY NO WARRANTY.
* IT PROVIDES THE NEED FOR CREATING THE EXECUTABLE FILES THAT HAVE BEEN USED
* BY THE AUTHORS FOR THE NUMERICAL EXPERIMENTS RECONSTRUCTIONS OF SECT. 4 
* AND 5 OF THE PAPER. 
*
* IT INHERITS THE SAME RIGHTS OF THE PUBLISHED PAPER. 
* IF YOU USE THIS CODE OF ANY PART OF IT, YOU ARE PLEASED TO QUOTE THE ARTICLE
* AND THE CODE NAME SuperFP (NOV. 2023)
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/


/* 
 * BUILD THE MATRIX OF THE CHANG-COOPER (CC) METHOD FOR A TWO
 * DIMENSIONAL MATRIX WITH ZERO FLUX BOUNDARY CONDITIONS
 * 
 */



#ifndef CHANG_COOPER_H
#define CHANG_COOPER_H


#include <armadillo>

#include "model.hpp"   




using namespace arma;



// ====================================================================

/* The Chang_Cooper class create an instance of the matrix of
 * the method, based on the model parameters and functions 
 */


class Chang_Cooper {
	
	
	/* The matrix follows the rules: the x-coordinate corresponds to 
	 * the row index, the y-coordinate to the coloumn, i.e. f[x,y].
	 * Hence the x-axis is alongside the coloumn, the y-axis alongside
	 * the rows.
	 */

	public:
		
		
	sp_mat CC_matrix;	// sparse matrix associated to the CC method.
						// the dimensionality is domain->NN
	
	// base constructors
	Chang_Cooper() {};
	
	// constructor joining to the model and numerical domain
	Chang_Cooper(Model* model, Grid* grid_domain);
	

	// build the matrix of the discrete operator of the CC method
	//
	void build_Chang_Cooper_matrix();	
	
	
	Model *md;		 // pointer to the model instance

	Grid  *domain;   // pointer to the structur of the square numerical domain
	
	
	private:	

		
	// evaluate the 'delta' parameter of the CC method
	//
	mat delta_CC(mat B, mat C);   
	
};

#endif
