/**********************************************************************+
*
* SuperFP: A program for the reconstruction of the driving potential 
*          of stochastic trajectories, by solving an inverse problem
*          governed by the corresponding Fokker-Planck equation
*
**********************************************************************+
*
* THIS PACKAGE OF PROGRAM FILES IS A SUPPLEMENTAL MATERIAL FOR THE MANUSCRIPT 
* "Fokker-Planck analysis of superresolution microscopy images"
* BY MARIO ANNUNZIATO AND ALFIO BORZI' (NOV. 2023).
*
* THIS SOFTWARE IS EXPERIMENTAL AND IS RELEASED WITH ABSOLUTELY NO WARRANTY.
* IT PROVIDES THE NEED FOR CREATING THE EXECUTABLE FILES THAT HAVE BEEN USED
* BY THE AUTHORS FOR THE NUMERICAL EXPERIMENTS RECONSTRUCTIONS OF SECT. 4 
* AND 5 OF THE PAPER. 
*
* IT INHERITS THE SAME RIGHTS OF THE PUBLISHED PAPER. 
* IF YOU USE THIS CODE OF ANY PART OF IT, YOU ARE PLEASED TO QUOTE THE ARTICLE
* AND THE CODE NAME SuperFP (NOV. 2023)
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/



/* This class defines the optimization problem */

#ifndef OPTIM_PROBL
#define OPTIM_PROBL

#include<armadillo>
#include<iostream>
#include "gradients.hpp"
#include "BDF_Chang_Cooper.hpp"


using namespace std;
using namespace arma;




// ==== PROBLEMA DI OTTIMIZZAZIONE ====

// ereditare da BDF_CC. dichiarare forward_solution e backw. protetti
// definire la funzione calc_forward_and_get_objective


/* Defines the class for the optimization problem. It inherit 
 * from the class of the numerical solver with BDF2 and Chang-Cooper
 */ 


 class Optim_problem : public BDF_CC { 

	// pointer to the discrete Laplace operator with Neumann boundary
	// conditions used for the resolution of the H1 gradient
	//
	sp_mat *Laplace_Neumann_mat=NULL;  
	
	cube *fd; 		  // pointer to the desired solution f_d
	cube source;	  // source term for the adjoint equation
	mat pT;           // terminal condition
	

	// gradients used for the calculation of the gradient of
	// the object functional
	//
	mat grad_L2();    // L2 gradient of the object functional
	mat grad_H1();	  // H1 gradient
	
	// build the matrix Laplace operator with Neumann buondary conditions
	sp_mat* build_Laplace_Neumann_matrix();
	
	
	// local implementation of the discrete Laplacian operator
	// (distinguishes the cases dx=dy and dx!=dy for futures implementations)
	mat discrete_Laplace();
	
	// evaluate the objective functional
	double objective();
	
	
	public:	
	
	// base constructor
	Optim_problem();
	
	// constructor joining to the model, numerical mesh and the desired solution
	Optim_problem(Model* md, Grid* grid,cube *p_fd);
	
	// destructor
	~Optim_problem();
	
	// evaluate the gradient of the objective functional
	mat calc_objective_gradient();  
	
	
	// calculate the forward solution and return the value of the objective functional
	double calc_forward_and_objective(mat& f0, mat &u);
		
	// calculate the adjoint solution (backward in time)								
	void calc_backward();
	
};


#endif
