/**********************************************************************+
*
* SuperFP: A program for the reconstruction of the driving potential 
*          of stochastic trajectories, by solving an inverse problem
*          governed by the corresponding Fokker-Planck equation
*
**********************************************************************+
*
* THIS PACKAGE OF PROGRAM FILES IS A SUPPLEMENTAL MATERIAL FOR THE MANUSCRIPT 
* "Fokker-Planck analysis of superresolution microscopy images"
* BY MARIO ANNUNZIATO AND ALFIO BORZI' (NOV. 2023).
*
* THIS SOFTWARE IS EXPERIMENTAL AND IS RELEASED WITH ABSOLUTELY NO WARRANTY.
* IT PROVIDES THE NEED FOR CREATING THE EXECUTABLE FILES THAT HAVE BEEN USED
* BY THE AUTHORS FOR THE NUMERICAL EXPERIMENTS RECONSTRUCTIONS OF SECT. 4 
* AND 5 OF THE PAPER. 
*
* IT INHERITS THE SAME RIGHTS OF THE PUBLISHED PAPER. 
* IF YOU USE THIS CODE OF ANY PART OF IT, YOU ARE PLEASED TO QUOTE THE ARTICLE
* AND THE CODE NAME SuperFP (NOV. 2023)
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/


/** Support functions for calculating the discrete gradients */

#ifndef GRADS_H
#define GRADS_H
  
#include <armadillo>

using namespace arma;



mat grad_x(mat* f, double dx);  ///< x-gradient of f. f is 2-dim
mat grad_y(mat* f, double dx);  ///< y-gradient of f. f is 2-dim

// overloaded functions for the 3-dimensional functions

cube grad_x(cube* f, double dx);  ///< x-gradient of f. f is 3-dim 

cube grad_y(cube* f, double dx);  ///< y-gradient of f. f is 3-dim


mat discrete_Laplace(mat* f, double dx, double dy); ///< Discrete Laplace operator

// overloaded version for the case dx=dy
//
mat discrete_Laplace(mat* f, double dx);///< Discrete Laplace operator with square mesh


#endif
