/**********************************************************************+
*
* SuperFP: A program for the reconstruction of the driving potential 
*          of stochastic trajectories, by solving an inverse problem
*          governed by the corresponding Fokker-Planck equation
*
**********************************************************************+
*
* THIS PACKAGE OF PROGRAM FILES IS A SUPPLEMENTAL MATERIAL FOR THE MANUSCRIPT 
* "Fokker-Planck analysis of superresolution microscopy images"
* BY MARIO ANNUNZIATO AND ALFIO BORZI' (NOV. 2023).
*
* THIS SOFTWARE IS EXPERIMENTAL AND IS RELEASED WITH ABSOLUTELY NO WARRANTY.
* IT PROVIDES THE NEED FOR CREATING THE EXECUTABLE FILES THAT HAVE BEEN USED
* BY THE AUTHORS FOR THE NUMERICAL EXPERIMENTS RECONSTRUCTIONS OF SECT. 4 
* AND 5 OF THE PAPER. 
*
* IT INHERITS THE SAME RIGHTS OF THE PUBLISHED PAPER. 
* IF YOU USE THIS CODE OF ANY PART OF IT, YOU ARE PLEASED TO QUOTE THE ARTICLE
* AND THE CODE NAME SuperFP (NOV. 2023)
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#include "model.hpp"
#include "gradients.hpp"







mat Model::sigma(uword nx, uword ny, double sig)
{

 mat s = ones<mat>(nx,ny);

 s = sig*s;


 return s;

};

mat Model::val_Bx(Grid* d, mat* uu)
{



 mat B = grad_x(uu,d->dx);

 return B;

}

mat Model::val_By(Grid* d, mat* uu)
{



 mat B = grad_y(uu,d->dy);

 return B;

}






mat Model::val_Cx(Grid* d)
{



 mat C = sigma(d->Nx-1,d->Ny,param.sig);

 C = C % C / 2.;

 return C;

}






mat Model::val_Cy(Grid* d)
{


 mat C = sigma(d->Nx,d->Ny-1,param.sig);

 C = C % C / 2.;

 return C;


};
